﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace PhotoViewer
{
    public class Photo : IComparable<Photo>, IEquatable<Photo>
    {
        public int Index
        {
            get;
            set;
        }

        public string Filename
        {
            get;
            set;
        }

        public string Description
        {
            get;
            set;
        }

        public Image PhotoImage
        {
            get;
            set;
        }

        public int Width
        {
            get;
            set;
        }

        public int Height
        {
            get;
            set;
        }

        public Photo() : this(0, null, null)
        {
        }

        public Photo(int index, string filename, string description)
        {
            this.Index = index;
            this.Filename = filename;
            this.Description = description;
            this.PhotoImage = Image.FromFile(this.Filename);
            this.Width = this.PhotoImage.Width;
            this.Height = this.PhotoImage.Height;
            //OutputDebugPrint();
        }

        public Photo(Photo other)
        {
            this.Index = other.Index;
            this.Filename = other.Filename;
            this.Description = other.Description;
            this.PhotoImage = other.PhotoImage;
            this.Width = other.Width;
            this.Height = other.Height;
        }

        // System.Object.ToString method override
        public override string ToString()
        {
            return string.Format("이름:{0} 크기:{1}x{2}", Description, Width, Height); // 화면 출력용
        }

        // debug method
        public void OutputDebugPrint()
        {
            System.Diagnostics.Trace.WriteLine(" 파일: " + Filename + " 설명: " + Description + " 크기 " + Width + " " + Height);
        }

        // needed for sort
        #region IComparable<Photo> Members
        public int CompareTo(Photo other)
        {
            return this.Index.CompareTo(other.Index);
        }

        public static Comparison<Photo> NameComparison = delegate(Photo p, Photo q)
        {
            return p.Filename.CompareTo(q.Filename);
        };
        #endregion

        // needed for search
        #region IEquatable<Photo> Members
        public override int GetHashCode()
        {
            return this.Index.GetHashCode();
        }

        public bool Equals(Photo other)
        {
            return this.Index.Equals(other.Index) && this.Description.Equals(other.Description);
        }
        #endregion

        // needed for ListView
        #region ListViewItem
        public ListViewItem ToListViewItem()
        {
            ListViewItem item = new ListViewItem(this.Filename);
            item.ImageIndex = this.Index;
            item.ImageKey = this.Filename;
            item.Text = this.Description;
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Index.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Description));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Width.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Height.ToString()));
            return item;
        }
        #endregion
    }
}
